@@ -7,36 +7,39 @@ function removeAllChildren(node) {
   }
 }
 
-function createEnvironmentDiv(key, description, path, checked = false) {
+function createEnvironmentDiv(key, description, path, modules, iscustom = false) {
   const div = document.createElement('div');
   div.classList.add('environment-div');
 
-  // Store env key in hidden input
-  const hidden_input = document.createElement('input');
-  hidden_input.setAttribute('type', 'hidden');
-  hidden_input.setAttribute('value', key);
-  div.appendChild(hidden_input);
-
   const radio_id = `environment_radio_${key}`;
-  const title = `Environment path: ${path}`;
+  const title = [path ? `Path: ${path}` : "", modules ? `Modules: ${modules}` : ""].filter(v => v).join("\n");
+
+  // Store definitions in hidden inputs
+  const path_input = document.createElement('input');
+  path_input.setAttribute('type', 'hidden');
+  path_input.setAttribute('class', 'environment_path');
+  path_input.setAttribute('value', iscustom ? path : '');
+  div.appendChild(path_input);
+  const mods_input = document.createElement('input');
+  mods_input.setAttribute('type', 'hidden');
+  mods_input.setAttribute('class', 'environment_modules');
+  mods_input.setAttribute('value', iscustom ? modules : '');
+  div.appendChild(mods_input);
 
   const input = document.createElement('input');
   input.setAttribute('type', 'radio');
   input.setAttribute('id', radio_id);
   input.setAttribute('title', title);
-  input.setAttribute('name', 'environment_path');
-  input.setAttribute('value', path);
+  input.setAttribute('name', 'environment_id');
+  input.setAttribute('value', key);
 
-  input.addEventListener('change', updateEnvironmentAddPathRequired);
-  if (checked) {
-    input.setAttribute('checked', '');
-  }
+  input.addEventListener('change', updateAddCustomEnvironment);
   div.appendChild(input);
 
   const label = document.createElement('label');
   label.setAttribute('for', radio_id);
   label.setAttribute('title', title);
-  label.textContent = description === '' ? path : description;
+  label.textContent = description ? description : [path, modules].filter(v => v).join(" - ");
   div.appendChild(label);
 
   return div;
@@ -56,13 +59,15 @@ function selectEnvironment(key) {
 
   if (key !== null) {
     const keyInputs = Array.from(
-      environmentsDiv.querySelectorAll('input[type="hidden"]')
+      environmentsDiv.querySelectorAll('input[type="radio"]')
     );
     const index = keyInputs.findIndex((element) => element.value === key);
     if (index >= 0) {
-      keyInputs[index].parentNode.querySelector(
-        'input[type="radio"]'
-      ).checked = true;
+      keyInputs[index].checked = true;
+      keyPath = keyInputs[index].parentNode.querySelector('input.environment_path');
+      document.getElementById('environment_path').value = keyPath ? keyPath.value : '';
+      keyModules = keyInputs[index].parentNode.querySelector('input.environment_modules');
+      document.getElementById('environment_modules').value = keyModules ? keyModules.value : '';
       environmentSimpleSelect.selectedIndex = index;
       return;
     }
@@ -77,16 +82,10 @@ function getSelectedEnvironment() {
   const selectedRadio = environmentsDiv.querySelector(
     'input[type="radio"]:checked'
   );
-  if (!selectedRadio) {
-    return null;
-  }
-  const hiddenInput = selectedRadio.parentNode.querySelector(
-    'input[type="hidden"]'
-  );
-  return hiddenInput ? hiddenInput.value : null;
+  return selectedRadio ? selectedRadio.value : null;
 }
 
-function addCustomEnvironment(key, description, path, persist = true) {
+function addCustomEnvironment(key, description, path, modules, persist = true) {
   const customEnvironmentDiv = document.getElementById(
     'jupyter_environments_custom'
   );
@@ -94,7 +93,7 @@ function addCustomEnvironment(key, description, path, persist = true) {
     'environment_simple_custom'
   );
 
-  const div = createEnvironmentDiv(key, description, path);
+  const div = createEnvironmentDiv(key, description, path, modules, true);
 
   button = document.createElement('button');
   button.setAttribute('type', 'button');
@@ -143,12 +142,12 @@ function removeCustomEnvironment(key, persist = true) {
     }
     option.parentNode.removeChild(option);
   }
-  const hiddenInput = customEnvironmentDiv.querySelector(
-    `input[type="hidden"][value=${key}]`
+  const radioSelect = customEnvironmentDiv.querySelector(
+    `input[type="radio"][value=${key}]`
   );
-  if (hiddenInput !== null) {
-    const div = hiddenInput.parentNode;
-    if (div.querySelector('input[type="radio"]').checked) {
+  if (radioSelect !== null) {
+    const div = radioSelect.parentNode;
+    if (radioSelect.checked) {
       resetEnvironmentSelection();
     }
     div.parentNode.removeChild(div);
@@ -170,11 +169,12 @@ function getCustomEnvironments() {
 
   const customEnvs = {};
 
-  // Get key from hidden input, description from label and path from radiobutton
+  // Get key from radio button, description from label and definitons in hidden input
   customEnvironmentDiv.querySelectorAll('.environment-div').forEach((div) => {
-    customEnvs[div.querySelector('input[type=hidden]').value] = {
+    customEnvs[div.querySelector('input[type=radio]').value] = {
       description: div.querySelector('label').textContent,
-      path: div.querySelector('input[type="radio"]').value,
+      path: div.querySelector('input.environment_path').value,
+      modules: div.querySelector('input.environment_modules').value,
     };
   });
   return customEnvs;
@@ -208,7 +208,7 @@ function restoreCustomEnvironmentsFromLocalStorage() {
   }
 
   for (const key in config) {
-    addCustomEnvironment(key, config[key].description, config[key].path, false);
+    addCustomEnvironment(key, config[key].description, config[key].path || "", config[key].modules || "", false);
   }
 }
 
@@ -233,7 +233,7 @@ function updateDefaultEnvironments() {
     const info = partitionInfo.jupyter_environments[key];
 
     defaultEnvironmentsDiv.appendChild(
-      createEnvironmentDiv(key, info.description, info.path)
+      createEnvironmentDiv(key, info.description, info.path, info.modules, false)
     );
 
     const option = document.createElement('option');
@@ -248,6 +248,7 @@ function resetSpawnForm() {
   document.getElementById('spawn_form').reset();
   restoreCustomEnvironmentsFromLocalStorage();
   setSimplePartition(window.SLURM_DATA.default_partition);
+  updateDefaultUrl();
 }
 
 function setVisible(element, visible) {
@@ -258,13 +259,21 @@ function setVisible(element, visible) {
   }
 }
 
-function updateEnvironmentAddPathRequired() {
+function updateAddCustomEnvironment() {
+  const environmentAddButton = document.getElementById('environment_add_button');
   const environmentAddRadio = document.getElementById('environment_add_radio');
   const environmentAddPath = document.getElementById('environment_add_path');
-  if (environmentAddRadio.checked) {
+  const environmentAddMods = document.getElementById('environment_add_modules');
+
+  const isPathOrModules = environmentAddPath.value !== "" || environmentAddMods.value !== "";
+  environmentAddButton.disabled = !isPathOrModules;
+
+  if (environmentAddRadio.checked && !isPathOrModules) {
     environmentAddPath.setAttribute('required', '');
+    environmentAddMods.setAttribute('required', '');
   } else {
     environmentAddPath.removeAttribute('required');
+    environmentAddMods.removeAttribute('required');
   }
 }
 
@@ -334,6 +343,13 @@ function updateMemValue() {
   memHiddenElem.value = value && value !== '0' ? `${value}G` : value;
 }
 
+function updateDefaultUrl() {
+  const defaultUrlCheckboxElem = document.getElementById('default_url');
+  const defaultUrlHiddenElem = document.getElementById('default_url_hidden_input');
+
+  defaultUrlHiddenElem.value = defaultUrlCheckboxElem.checked ? '/lab' : '/tree';
+}
+
 function updatePartitionLimits() {
   const nprocsElem = document.getElementById('nprocs');
   const nprocsSpanElem = document.getElementById('max_nprocs_span');
@@ -470,10 +486,11 @@ document.addEventListener('DOMContentLoaded', () => {
   const nprocsElem = document.getElementById('nprocs');
   const ngpusElem = document.getElementById('ngpus');
   const runtimeElem = document.getElementById('runtime');
-  const default_url = document.getElementById('default_url');
+  const defaultUrlCheckboxElem = document.getElementById('default_url');
   const environmentAddRadio = document.getElementById('environment_add_radio');
   const environmentAddName = document.getElementById('environment_add_name');
   const environmentAddPath = document.getElementById('environment_add_path');
+  const environmentAddMods = document.getElementById('environment_add_modules');
   const environmentAddButton = document.getElementById(
     'environment_add_button'
   );
@@ -505,8 +522,11 @@ document.addEventListener('DOMContentLoaded', () => {
   document
     .getElementById('jupyterlab_simple')
     .addEventListener('change', (e) => {
-      default_url.checked = e.target.checked;
+      defaultUrlCheckboxElem.checked = e.target.checked;
+      updateDefaultUrl();
     });
+  // Update default_url hidden input
+  defaultUrlCheckboxElem.addEventListener('change', updateDefaultUrl)
   // Runtime
   document.getElementById('runtime_simple').addEventListener('change', (e) => {
     runtimeElem.value = `${e.target.value}:00:00`;
@@ -537,27 +557,39 @@ document.addEventListener('DOMContentLoaded', () => {
     .getElementById('environment_simple')
     .addEventListener('change', (e) => selectEnvironment(e.target.value));
 
+  // Handle update of environment advanced
+  document
+    .getElementById('jupyter_environments_default')
+    .addEventListener('change', (e) => selectEnvironment(e.target.value));
+  document
+    .getElementById('jupyter_environments_custom')
+    .addEventListener('change', (e) => selectEnvironment(e.target.value));
+
   // Update mem when mem_input changes
   document
     .getElementById('mem_input')
     .addEventListener('change', (e) => updateMemValue());
 
   // Handle add custom environment
-  document
-    .getElementById('environment_add_path')
-    .addEventListener('input', (e) => {
-      const isInputEmpty = e.target.value === '';
-      if (environmentAddRadio.value === '' && !isInputEmpty) {
-        // First input in the path: select its radio button
-        environmentAddRadio.checked = true;
-      }
-      environmentAddRadio.value = e.target.value;
-      environmentAddButton.disabled = isInputEmpty;
-    });
+  environmentAddPath.addEventListener('input', updateAddCustomEnvironment);
+  environmentAddMods.addEventListener('input', updateAddCustomEnvironment);
+
+  environmentAddName.addEventListener('focus', () => {
+    environmentAddRadio.checked = true;
+    updateAddCustomEnvironment();
+  });
+  environmentAddPath.addEventListener('focus', () => {
+    environmentAddRadio.checked = true;
+    updateAddCustomEnvironment();
+  });
+  environmentAddMods.addEventListener('focus', () => {
+    environmentAddRadio.checked = true;
+    updateAddCustomEnvironment();
+  });
 
   environmentAddRadio.addEventListener(
     'change',
-    updateEnvironmentAddPathRequired
+    updateAddCustomEnvironment
   );
 
   environmentAddButton.addEventListener('click', (e) => {
@@ -565,15 +597,16 @@ document.addEventListener('DOMContentLoaded', () => {
     addCustomEnvironment(
       key,
       environmentAddName.value,
-      environmentAddPath.value
+      environmentAddPath.value,
+      environmentAddMods.value
     );
     if (environmentAddRadio.checked) {
       selectEnvironment(key);
-      updateEnvironmentAddPathRequired();
     }
     environmentAddName.value = '';
     environmentAddPath.value = '';
-    environmentAddPath.dispatchEvent(new Event('input'));
+    environmentAddMods.value = '';
+    updateAddCustomEnvironment();
   });
 
   // Catch form submit
